;-------------------------------------------------------------------
;	PROGRAM NAME & REVISION INFORMATION
;
;-------------------------------------------------------------------
;	CALIBRATION BIT TEST & CALIBRATE
;
;	VERSION 1.0 
;
;	DESIGNED BY E BROUWER
;
;	DATE: 23 JULY 2012

;-------------------------------------------------------------------
;	DESCRIPTION

;-------------------------------------------------------------------
;	VERIFY IF CHIP CONTAINS VALID CALIBRATION BIT
;	- TURN ON GREEN LED IF CALIBRATION DATA PRESENT
;	- TURN ON RED LED IF NOT
;   - TURN ON GREEN & RED LED IF CALIBRATION DATA PROGRAMMED INTO EEPROM
;	
;-------------------------------------------------------------------
;	PIN ASSIGNMENTS
;
;-------------------------------------------------------------------
;	PIN	INPUT/OUTPUT	USED FOR
;	GP0	OUTPUT		GREEN LED - CALIBRATION BIT PRESENT
;	GP1	OUTPUT		RED LED   - NO CALIBRATION BIT
;	GP2	OUTPUT		1KHZ TEST SIGNAL
;	GP3	INPUT		LO TO PROGRAM CALIBRRATION DATA INTO EEPROM
;	GP4	INPUT		LO TO LOWER SPEED
;	GP5	INPUT		LO TO RAISE SPEED
;
;-------------------------------------------------------------------
;	COMPILER SETTINGS
;
;-------------------------------------------------------------------
 	LIST    F=INHX8M, P=12F675, R=HEX
 	#include <p12F675.inc>  

	;CONFIGRATION BITS
	;-----------------
	__CONFIG _INTRC_OSC_NOCLKOUT & _WDT_OFF &  _PWRTE_ON & _MCLRE_ON &  _BODEN_OFF & _CP_OFF & _CPD_OFF 

;-------------------------------------------------------------------
;	VARIABLES/LABELS DECLARATIONS
;
;-------------------------------------------------------------------
	CBLOCK H'20'		;DEFINE RAM LOCATION
		CAL_BIT		;CALIBRATION DATA
		DLY_1		;DELAY
		DLY_2		;DELAY
		DLY_3
		DLY_4
	ENDC

#DEFINE	GN_LED	GPIO,0 		
#DEFINE	OUTPUT	GPIO,1		
#DEFINE	BTN_PGM	GPIO,2
#DEFINE	BTN_DN	GPIO,4	
#DEFINE	BTN_UP	GPIO,5	


;-------------------------------------------------------------------
;	START-UP OF PIC 
;
;-------------------------------------------------------------------
	ORG	0 		;RESET VECTOR
	GOTO	START		;	/
	ORG	4		;INTERRUPT VECTOR
	GOTO	ISR		;	/
	
;-------------------------------------------------------------------
;	SET UP REGISTERS
;
;-------------------------------------------------------------------
START	
	CLRF	GPIO		;CLEAR PORTS
	MOVLW	B'111'		;SET COMPARATOR TO OFF
	MOVWF	CMCON		;	/
	BSF	STATUS,RP0	;SET BANK 1
	CLRF	ANSEL		;SELECT ALL DIGITAL I/O
	MOVLW	B'111100'		;SELECT PULLUP RESISTORS
	MOVWF	WPU		;	/
	MOVLW	B'111100'		;SETUP IO DIRECTION
	MOVWF	TRISIO		;	/
	MOVLW	B'00010111'	;LOAD OPTION REGISTER
				;BIT 7 - 0 = WEAK PULL-UPS ENABLED
				;BIT 6 - 0 = INTERRUPT ON RISING EDGE OF GP2
				;BIT 5 - 1 = TMR0 CLOCK SET TO GP2 PIN
				;BIT 4 - 1 = TMR0 INC ON HIGH TO LOW 
				;BIT 3 - 1 = PRESCALER ASSIGNED TO WDT
				;BIT 2..1  = PRESCALER SET TO MAX
	MOVWF	OPTION_REG	;	/
	BCF	STATUS,RP0	;SET BANK 0
	
	; 500ms STARTUP DELAY
	;--------------------
	MOVLW	H'5'
	MOVWF	DLY_4
START1	MOVLW	H'FF'
	MOVWF	DLY_3
	DECFSZ	DLY_4,1
	GOTO	START2
	GOTO	START3
START2	CALL	DELAY		;500us DELAY
	DECFSZ	DLY_3,1
	GOTO	START2
	GOTO	START1
START3	CALL	WRITE_EE		;WRITE DEFAULT EEDATA

;-------------------------------------------------------------------
;	CHECK IF CALIBRATION BIT IS PRESENT
;
;-------------------------------------------------------------------

	BTFSC	INTCON,0		;IS 0 ON POWER UP
				;IS 1 AFTER RESET (NO CAL BIT)
	GOTO	NO_CAL		;NO CALIBRTION DATA AVAILABLE
				;TRY TO READ CALIBRATION DATA
	BSF	INTCON,0		;SET INTCON,0 TO INDICATE READING OF CALIBRATION DATA
	CALL	H'3FF'		;LOAD CALIBRATION DATA
	BCF	INTCON,0		;END OF CALIBRATION ROUTINE
				;WILL RESTART PIC IF NO CALIBRATION DATA AVAILABLE
	;VALID CALIBRATION BIT AVAILABLE
	;-------------------------------
	;LOAD CALIBRATION DATA INTO OSCCAL
	;---------------------------------
	CALL	SAVE_CAL		;SAVE CAL BIT IN EEDATA (LOCATION H'6F')
	BSF	STATUS,RP0	;SET BANK 1
	MOVLW	H'00'
	MOVWF	OSCCAL		;SET CALIRATION
	BCF	STATUS,RP0	;SET BANK 0
	GOTO	START_OK

;-------------------------------------------------------------------
;	CHECK IF CALIBRATION EXIST IN EEPROM (LOCATION H'7F')
;	
;-------------------------------------------------------------------
NO_CAL	
	BCF	INTCON,0		;RESET CALIBRATION DATA FLAG
	MOVLW	H'00'
          CALL	SAVE_CAL
	
;-------------------------------------------------------------------
;	STARTUP COMPLETE
;	
;-------------------------------------------------------------------
START_OK
	NOP


;-------------------------------------------------------------------
;	MAIN PROGRAM LOOP
;	OUTPUT 1KHz TEST SIGNAL ON GPIO,0
;-------------------------------------------------------------------
MAIN	
	BTFSS	BTN_UP		;IS UP BUTTON PRESSED?
	CALL	BTN_UP1		;YES
	BTFSS	BTN_DN		;IS DOWN BUTTON PRESSED?
	CALL	BTN_DN1		;YES
	BTFSS	BTN_PGM		;IS PROGRAM BUTTON PRESSED?
	CALL	BTN_PGM1		;YES
	NOP			;GET TIMING CORRECT
	NOP			;	/
	NOP			;	/

	;1KHz OUTPUT PULSE
	;-----------------
	BSF	OUTPUT		;TEST OUTPUT HIGH
	CALL	DELAY		;500us DELAY

	NOP			;GET TIMING CORRECT
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	NOP			;	/
	
	BCF	OUTPUT		;TEST OUTPUT LO
	CALL	DELAY		;500us DELAY

	GOTO	MAIN

BTN_UP1	
	;READ OSCCAL
	;-----------
	BSF	STATUS,RP0	;SET BANK 1
	MOVF	OSCCAL,W		;READ OSCCAL
	;ADD 4 TO OSCCAL
	;---------------
	ADDLW	D'4'		;ADD 4 TO OSCCAL
	MOVWF	OSCCAL		;SET CALIRATION
	BCF	STATUS,RP0	;SET BANK 0
BTN_UP2	BTFSS	BTN_UP		;WAIT FOR BUTTON TO BE RELEASED
	GOTO	BTN_UP2		;	/
	RETURN

BTN_DN1	
	;READ OSCCAL
	;-----------
	BSF	STATUS,RP0	;SET BANK 1
	MOVF	OSCCAL,W		;READ OSCCAL
	;SUBTRACT 4 FROM OSCCAL
	;----------------------
	MOVLW	D'4'		;W = 4
	SUBWF	OSCCAL,0		;SUBTRACT 4 FROM OSCCAL
	MOVWF	OSCCAL		;SET CALIRATION
	BCF	STATUS,RP0	;SET BANK 0
BTN_DN2	BTFSS	BTN_DN		;WAIT FOR BUTTON TO BE RELEASED
	GOTO	BTN_DN2		;	/
	RETURN

BTN_PGM1	
	;SAVE COPY IN EEPROM (LOCATION H'7F')
	;------------------------------------
	CALL	SAVE_CAL1		;SAVE CALIRATION DATA
	BSF	GN_LED		;TURN GREEN LED ON

BTN_PGM2	BTFSS	BTN_PGM		;WAIT FOR BUTTON TO BE RELEASED
	GOTO	BTN_PGM2		;	/
	RETURN


;-------------------------------------------------------------------
;	SAVE ORIGINAL CALIBRATION DATA IN EEPROM
;	
;-------------------------------------------------------------------
SAVE_CAL	BSF	STATUS,RP0	;SET BANK 1
	MOVWF	EEDATA		;INTO EEDATA REGISTER
	MOVLW	H'6F'		;SET EEPROM ADDRESS
	MOVWF	EEADR		;	/
	
	;SAVE DATA INTO EEPROM
	;---------------------
	BSF	EECON1,WREN	;ENABLE WRITE
	MOVLW	H'55'		;UNLOCK WRITE
	MOVWF	EECON2		;	/
	MOVLW	H'AA'		;	/
	MOVWF	EECON2		;	/
	BSF	EECON1,WR		;START THE WRITE
EE1	BTFSC	EECON1,WR		;IS WRITE COMPLETE
	GOTO	EE1		;NOT YET
	BCF	STATUS,RP0	;SET BANK 0
	RETURN			;DONE

;-------------------------------------------------------------------
;	SAVE NEW CALIBRATION DATA IN EEPROM
;	
;-------------------------------------------------------------------
SAVE_CAL1	BSF	STATUS,RP0	;SET BANK 1
	MOVF	OSCCAL,W		;READ OSCCAL
	MOVWF	EEDATA		;INTO EEDATA REGISTER
	MOVLW	H'7F'		;SET EEPROM ADDRESS
	MOVWF	EEADR		;	/

	;SAVE DATA INTO EEPROM
	;---------------------
	BSF	EECON1,WREN	;ENABLE WRITE
	MOVLW	H'55'		;UNLOCK WRITE
	MOVWF	EECON2		;	/
	MOVLW	H'AA'		;	/
	MOVWF	EECON2		;	/
	BSF	EECON1,WR		;START THE WRITE
EE2	BTFSC	EECON1,WR		;IS WRITE COMPLETE
	GOTO	EE2		;NOT YET
	BCF	STATUS,RP0	;SET BANK 0
	RETURN			;DONE

;-------------------------------------------------------------------
;	1ms SEC DELAY
;	
;-------------------------------------------------------------------
DELAY	MOVLW	D'1'  		;SET 50ms DELAY HERE (50)
	MOVWF	DLY_1
D_1	MOVLW	D'117'		;50 IS ABOUT 50ms
	MOVWF	DLY_2
D_2		NOP
	DECFSZ	DLY_2,1
	GOTO	D_2
	DECFSZ	DLY_1,1
	GOTO	D_1
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	RETURN

;-------------------------------------------------------------------
;	INTERRUPT SERVICE ROUTINE
;
;-------------------------------------------------------------------
ISR	
	RETFIE

;-------------------------------------------------------------------
;	SAVE DEFAULT EEDATA
;	
;-------------------------------------------------------------------
WRITE_EE	BSF	STATUS,RP0	;SET BANK 1
	MOVLW	H'3C'
	MOVWF	EEDATA
	MOVLW	H'60'	
	CALL	W_E	

	MOVLW	H'2D'
	MOVWF	EEDATA
	MOVLW	H'61'	
	CALL	W_E	

	MOVLW	H'2D'
	MOVWF	EEDATA
	MOVLW	H'62'	
	CALL	W_E	

	MOVLW	H'43'
	MOVWF	EEDATA
	MOVLW	H'63'	
	CALL	W_E	

	MOVLW	H'48'
	MOVWF	EEDATA
	MOVLW	H'64'	
	CALL	W_E	

	MOVLW	H'49'
	MOVWF	EEDATA
	MOVLW	H'65'	
	CALL	W_E	

	MOVLW	H'50'
	MOVWF	EEDATA
	MOVLW	H'66'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'67'	
	CALL	W_E	

	MOVLW	H'4F'
	MOVWF	EEDATA
	MOVLW	H'68'	
	CALL	W_E	

	MOVLW	H'53'
	MOVWF	EEDATA
	MOVLW	H'69'	
	CALL	W_E	

	MOVLW	H'43'
	MOVWF	EEDATA
	MOVLW	H'6A'	
	CALL	W_E	

	MOVLW	H'41'
	MOVWF	EEDATA
	MOVLW	H'6B'	
	CALL	W_E	

	MOVLW	H'4C'
	MOVWF	EEDATA
	MOVLW	H'6C'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'6D'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'6E'	
	CALL	W_E	

	MOVLW	H'00'
	MOVWF	EEDATA
	MOVLW	H'6F'	
	CALL	W_E	



	MOVLW	H'3C'
	MOVWF	EEDATA
	MOVLW	H'70'	
	CALL	W_E	

	MOVLW	H'2D'
	MOVWF	EEDATA
	MOVLW	H'71'	
	CALL	W_E	

	MOVLW	H'2D'
	MOVWF	EEDATA
	MOVLW	H'72'	
	CALL	W_E	

	MOVLW	H'4E'
	MOVWF	EEDATA
	MOVLW	H'73'	
	CALL	W_E	

	MOVLW	H'45'
	MOVWF	EEDATA
	MOVLW	H'74'	
	CALL	W_E	

	MOVLW	H'57'
	MOVWF	EEDATA
	MOVLW	H'75'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'76'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'77'	
	CALL	W_E	

	MOVLW	H'4F'
	MOVWF	EEDATA
	MOVLW	H'78'	
	CALL	W_E	

	MOVLW	H'53'
	MOVWF	EEDATA
	MOVLW	H'79'	
	CALL	W_E	

	MOVLW	H'43'
	MOVWF	EEDATA
	MOVLW	H'7A'	
	CALL	W_E	

	MOVLW	H'41'
	MOVWF	EEDATA
	MOVLW	H'7B'	
	CALL	W_E	

	MOVLW	H'4C'
	MOVWF	EEDATA
	MOVLW	H'7C'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'7D'	
	CALL	W_E	

	MOVLW	H'20'
	MOVWF	EEDATA
	MOVLW	H'7E'	
	CALL	W_E
	
	MOVLW	H'00'
	MOVWF	EEDATA
	MOVLW	H'7F'	
	CALL	W_E	

	BCF	STATUS,RP0	;SET BANK 0
	RETURN
W_E
	;SAVE DATA INTO EEPROM
	;---------------------
	MOVWF	EEADR		;	/
	BSF	EECON1,WREN	;ENABLE WRITE
	MOVLW	H'55'		;UNLOCK WRITE
	MOVWF	EECON2		;	/
	MOVLW	H'AA'		;	/
	MOVWF	EECON2		;	/
	BSF	EECON1,WR		;START THE WRITE
EE3	BTFSC	EECON1,WR		;IS WRITE COMPLETE
	GOTO	EE3		;NOT YET
	RETURN			;DONE



;-------------------------------------------------------------------
;	END OF PROGRAM
;	
;-------------------------------------------------------------------
	END
